import * as d3 from "d3";
import { BasePlot, BasePlotItem } from "./base";
import { PlotSelect, Scale, Format } from "./aliases";
export declare type Point = [number, number];
export declare type SPoint = [number, number, number];
/** The base data type for adding data to an evolution plot. */
export declare abstract class EvolutionItem extends BasePlotItem {
    protected lab: string;
    protected cve: d3.CurveFactory;
    protected pnt: d3.Symbol<any, any>;
    constructor();
    /** Get or set the label for the data item */
    label(): string;
    label(lab: string): this;
    /** Get or set the curve for the data item */
    curve(): d3.CurveFactory;
    curve(cve: d3.CurveFactory): this;
    /** Get or set the point for the data item */
    point(): d3.Symbol<any, any>;
    point(pnt: d3.Symbol<any, any>): this;
    /** Where to plot the label */
    abstract target(): number;
    /** What this items name is */
    abstract name(): string;
    /** Plot the data item */
    plot(group: PlotSelect, x: Scale, y: Scale): PlotSelect;
}
/** A line */
export declare class Line extends EvolutionItem {
    private readonly data;
    constructor(data: Point[]);
    target(): number;
    name(): string;
    plot(group: PlotSelect, x: Scale, y: Scale): PlotSelect;
}
/** A span representing the evolution of a range */
export declare class Span extends EvolutionItem {
    private readonly data;
    constructor(data: SPoint[]);
    target(): number;
    name(): string;
    plot(group: PlotSelect, x: Scale, y: Scale): PlotSelect;
}
/** Plot an evolution */
export declare class Evolution extends BasePlot {
    private wdth;
    private hght;
    private data;
    private xMin;
    private xMinSet;
    private xMax;
    private xMaxSet;
    private yMin;
    private yMinSet;
    private yMax;
    private yMaxSet;
    private xScale;
    private yScale;
    private xLabel;
    private yLabel;
    private xTicks;
    private yTicks;
    private xTickFormat;
    private yTickFormat;
    constructor(width: number, height: number);
    /** Add a line to the evolution */
    line(data: Point[]): Line;
    /** Add a span to the evolution */
    span(data: SPoint[]): Span;
    /** Set the plot width */
    width(width: number): this;
    /** Set the plot height */
    height(height: number): this;
    /** Get or set the minimum x value
     *
     * Once set, this value will not be updated by new data.
     */
    xmin(): number;
    xmin(min: number): this;
    /** Get or set the maximum x value
     *
     * Once set, this value will not be updated by new data.
     */
    xmax(): number;
    xmax(max: number): this;
    /** Get or set the x bounds
     *
     * This is a convenience method for xmin and xmax together.
     */
    xbounds(): [number, number];
    xbounds(bounds: [number, number]): this;
    /** Get or set the minimum y value
     *
     * Once set, this value will not be updated by new data.
     */
    ymin(): number;
    ymin(min: number): this;
    /** Get or set the maximum y value
     *
     * Once set, this value will not be updated by new data.
     */
    ymax(): number;
    ymax(max: number): this;
    /** Get or set the y bounds
     *
     * This is a convenience method for ymin and ymax together.
     */
    ybounds(): [number, number];
    ybounds(bounds: [number, number]): this;
    /** Set properties of the x axis */
    xaxis({label, ticks, format, scale}: {
        label?: string;
        ticks?: number[];
        format?: Format;
        scale?: d3.ScaleContinuousNumeric<number, number>;
    }): this;
    /** Set properties of the y axis */
    yaxis({label, ticks, format, scale}: {
        label?: string;
        ticks?: number[];
        format?: Format;
        scale?: d3.ScaleContinuousNumeric<number, number>;
    }): this;
    name(): string;
    /** Plot an evolution */
    plot(svgElement: SVGSVGElement): PlotSelect;
}
